include("script/campaign/libcampaign.js");
include("script/campaign/templates.js");
var consoleVar;
var tutState;
var didTheyHelpBuildGen;

//Alias for button
const CLOSE_BUTTON = 0;
const PRODUCTION_BUTTON = 1;
const RESEARCH_BUTTON = 2;
const BUILD_BUTTON = 3;
const DESIGN_BUTTON = 4;
const INTEL_BUTTON = 5;
const COMMAND_BUTTON = 6;

//consoleVar is an array of objects used to hold information about the following:
//1. audio: What sound file to play.
//2. clear: Should the console be cleared now.
//3. message: What to display in the console.
//4. state: What phase in the tutorial we need to be in to use this object
//5. wait: how long to wait, in seconds, before using this object. This is volatile and changes to zero later.
//6. func: a function to execute.
function setUpConsoleAndAudioVar()
{
	consoleVar = [
		{"audio": "tut1.ogg", "clear": false, "message": _("Build a resource extractor"), "state": 0, "wait": 1},
		{"audio": "tut2.ogg", "clear": false, "message": _("Select one of your trucks"), "state": 0, "wait": 3},
		{"audio": "tut4.ogg", "clear": false, "message": _("Now select the oil pool indicated by the radar pulse"), "state": 1, "wait": 1},
		{"audio": "tut5.ogg", "clear": false, "message": _("To convert the resources into power build a power generator"), "state": 2, "wait": 2},
		{"audio": "tut7.ogg", "clear": false, "message": _("Select the build icon"), "state": 2, "wait": 4, "func": "enableBuild"},
		{"audio": "tut8.ogg", "clear": false, "message": _("Now select the power generator in the build menu"), "state": 3, "wait": 0},
		{"audio": "tut9.ogg", "clear": false, "message": _("Position the grid at your base"), "state": 4, "wait": 0},
		{"audio": "tut10.ogg", "clear": false, "message": _("To increase your build rate, select your second truck"), "state": 5, "wait": 0},
		{"audio": "tut11.ogg", "clear": false, "message": _("Now select the power generator site"), "state": 6, "wait": 2},
		{"audio": "tut12.ogg", "clear": false, "message": _("The truck will now help build the power generator"), "state": 7, "wait": 2},
		//==PART TWO== Power Station has been built, by whatever means
		{"audio": "tut13.ogg", "clear": false, "message": _("Use a truck to search for the artifact indicated by the radar pulse"), "state": 8, "wait": 3, "func": "addTheArtifact"},
		{"audio": "tut14.ogg", "clear": false, "message": _("Use the truck to recover the artifact"), "state": 8, "wait": 4},
		{"audio": "tut7.ogg", "clear": false, "message": _("Select the build icon"), "state": 9, "wait": 3},
		{"audio": "tut15.ogg", "clear": false, "message": _("Now select the research facility and position it at your base"), "state": 10, "wait": 1},
		{"audio": "tut16.ogg", "clear": false, "message": _("Use your other truck to help build the facility"), "state": 11, "wait": 0},
		//==PART THREE== they've built a research centre!!! Whahey!
		{"audio": "tut17.ogg", "clear": false, "message": _("Select the research icon"), "state": 12, "wait": 0, "func": "enableRes"},
		{"audio": "tut18.ogg", "clear": false, "message": _("Now select the artifact"), "state": 13, "wait": 0},
		{"audio": "tut19.ogg", "clear": false, "message": _("The artifact will now be researched"), "state": 14, "wait": 0},
		{"audio": "tut20.ogg", "clear": false, "message": _("Design a combat vehicle by selecting the design icon"), "state": 15, "wait": 1},
		{"audio": undefined, "clear": false, "message": _("Select the new design button"), "state": 16, "wait": 2},
		{"audio": undefined, "clear": true, "message": _("Select the Vehicle Body button"), "state": 16, "wait": 3},
		{"audio": undefined, "clear": false, "message": _("Now select the Viper Body"), "state": 16, "wait": 3},
		{"audio": undefined, "clear": false, "message": _("Select the Vehicle Propulsion button"), "state": 17, "wait": 1},
		{"audio": undefined, "clear": false, "message": _("Now select the Wheeled Propulsion"), "state": 17, "wait": 2},
		{"audio": undefined, "clear": false, "message": _("Select the Vehicle Turret button"), "state": 18, "wait": 1},
		{"audio": undefined, "clear": false, "message": _("Now select the Machine Gun Turret"), "state": 18, "wait": 2},
		{"audio": undefined, "clear": false, "message": undefined, "state": 18, "wait": 2},
		{"audio": undefined, "clear": false, "message": undefined, "state": 18, "wait": 3},
		{"audio": undefined, "clear": false, "message": _("Design Complete. Select close button"), "state": 19, "wait": 3},
		//==PART FOUR== they've designed their first droid!!!
		{"audio": "tut7.ogg", "clear": false, "message": _("Select the build icon"), "state": 20, "wait": 3},
		{"audio": "tut21.ogg", "clear": false, "message": _("Select the factory and position it at your base"), "state": 21, "wait": 3},
		{"audio": "tut22.ogg", "clear": false, "message": _("The factory's delivery point is indicated by the number 1"), "state": 23, "wait": 2},
		{"audio": undefined, "clear": false, "message": _("This can be moved by selecting it then selecting where you want to position it"), "state": 23, "wait": 3},
		{"audio": "tut23.ogg", "clear": false, "message": _("Select the manufacture icon"), "state": 24, "wait": 2},
		{"audio": "tut24.ogg", "clear": false, "message": _("Now in the factories build list select the combat vehicle"), "state": 25, "wait": 2},
		{"audio": undefined, "clear": false, "message": _("Please wait while factory manufactures vehicle"), "state": 25, "wait": 3},
		//==PART FIVE== They've built their first droid.
		{"audio": "tut25.ogg", "clear": false, "message": undefined, "state": 26, "wait": 1},
	];
}

function increaseTutorialState()
{
	tutState = tutState + 1;
}

function grantStartTech()
{
	const TECH = [
		"R-Vehicle-Body01", "R-Sys-Spade1Mk1", "R-Vehicle-Prop-Wheels"
	];
	const STRUCTS = [
		"A0CommandCentre", "A0PowerGenerator", "A0ResourceExtractor",
		"A0ResearchFacility", "A0LightFactory"
	];

	camCompleteRequiredResearch(TECH, CAM_HUMAN_PLAYER);
}

function eventDroidBuilt()
{
	if (tutState === 25)
	{
		increaseTutorialState();
		//removeReticuleButton(PRODUCTION_BUTTON);
	}
}

function eventDeliveryPointMoved()
{
	if (tutState === 23)
	{
		increaseTutorialState();
		setReticuleButton(1, _("Manufacture (F1)"), "image_manufacture_up.png", "image_manufacture_down.png");
		setReticuleFlash(PRODUCTION_BUTTON, true);
	}
}

function eventMenuManufacture()
{
	if (tutState === 24)
	{
		increaseTutorialState();
		setReticuleFlash(PRODUCTION_BUTTON, false);
	}
}

//show the build button
function enableBuild()
{
	setReticuleButton(BUILD_BUTTON, _("Build (F3)"), "image_build_up.png", "image_build_down.png");
	setReticuleFlash(BUILD_BUTTON, true);
	enableStructure("A0PowerGenerator", CAM_HUMAN_PLAYER);
}

//show the research button
function enableRes()
{
	setReticuleButton(2, _("Research (F2)"), "image_research_up.png", "image_research_down.png");
	setReticuleFlash(RESEARCH_BUTTON, true);
	enableStructure("A0ResearchFacility", CAM_HUMAN_PLAYER);
}

//They clicked the button so turn off flash
function eventMenuBuild()
{
	if (tutState === 2) //power generator
	{
		increaseTutorialState();
		setReticuleFlash(BUILD_BUTTON, false);
	}
	else if (tutState === 9) // research facility
	{
		increaseTutorialState();
		setReticuleFlash(BUILD_BUTTON, false);
	}
	else if (tutState === 20)
	{
		increaseTutorialState();
		setReticuleFlash(BUILD_BUTTON, false);
	}
}

//They clicked the button so turn off flash
function eventMenuResearch()
{
	if (tutState === 12)
	{
		increaseTutorialState();
		setReticuleFlash(RESEARCH_BUTTON, false);
	}
}

function eventMenuBuildSelected()
{
	if (tutState === 3)
	{
		increaseTutorialState();
		checkForPowGen();
	}
	else if (tutState === 10)
	{
		increaseTutorialState();
		checkResFac();
	}
	else if (tutState === 21)
	{
		increaseTutorialState();
	}
}

function eventResearched(research)
{
	if (tutState === 14)
	{
		increaseTutorialState();
		setReticuleButton(DESIGN_BUTTON, _("Design (F4)"), "image_design_up.png", "image_design_down.png");
		setReticuleFlash(DESIGN_BUTTON, true);
	}
}

function eventMenuResearchSelected()
{
	if (tutState === 13)
	{
		increaseTutorialState();
		//removeReticuleButton(RESEARCH_BUTTON);
	}
}

function eventMenuDesign()
{
	if (tutState === 15)
	{
		increaseTutorialState();
		setReticuleFlash(DESIGN_BUTTON, false);
	}
}

function eventDesignBody()
{
	if (tutState === 16)
	{
		increaseTutorialState();
	}
}

function eventDesignPropulsion()
{
	if (tutState === 17)
	{
		increaseTutorialState();
	}
}

function eventDesignWeapon()
{
	if (tutState === 18)
	{
		increaseTutorialState();
		setReticuleFlash(CLOSE_BUTTON, true);
	}
}

function eventDesignQuit()
{
	if (tutState === 19)
	{
		increaseTutorialState();
		//removeReticuleButton(DESIGN_BUTTON);
		enableStructure("A0LightFactory", CAM_HUMAN_PLAYER);
		setReticuleFlash(CLOSE_BUTTON, false);
		setReticuleButton(3, _("Build (F3)"), "image_build_up.png", "image_build_down.png");
		setReticuleFlash(BUILD_BUTTON, true);
		enableTemplate("ConstructionDroid");
	}
}

//They built the power gen at tutState 4.
function checkForPowGen()
{
	if (tutState === 4)
	{
		if (countStruct("A0PowerGenerator", CAM_HUMAN_PLAYER) > 0)
		{
			//removeReticuleButton(BUILD_BUTTON);
			increaseTutorialState();
		}
		else
		{
			queue("checkForPowGen", 150);
		}
	}
}

function checkResFac()
{
	if (tutState === 11)
	{
		if (countStruct("A0ResearchFacility", CAM_HUMAN_PLAYER) > 0)
		{
			//removeReticuleButton(BUILD_BUTTON);
		}
		else
		{
			queue("checkResFac", 150);
		}
	}
}

//They need to get the second truck to help build the generator
function checkHelpBuild()
{
	if (tutState === 6)
	{
		var objects = enumDroid(CAM_HUMAN_PLAYER);
		for (var i = 0, l = objects.length; i < l; ++i)
		{
			var obj = objects[i];
			if (obj.type === DROID && obj.droidType === DROID_CONSTRUCT && obj.order === DORDER_HELPBUILD)
			{
				increaseTutorialState();
				didTheyHelpBuildGen = true;
				break;
			}
		}

		queue("checkHelpBuild", 150);
	}
}

//See local eventPickup next.
function addTheArtifact()
{
	camSetArtifacts({ "mgArtifactPos": { tech: "R-Wpn-MG1Mk1" } });
}

function eventPickup(feature, droid)
{
	if (tutState === 8)
	{
		increaseTutorialState();
		setReticuleFlash(BUILD_BUTTON, true);
		enableStructure("A0ResearchFacility", CAM_HUMAN_PLAYER);
	}
}

//automatically plays audio and displays console text when the correct state is achieved
//by requeueing itself.
function addToConsole()
{
	if (consoleVar.length)
	{
		var tutPhase = consoleVar[0];
		if (tutPhase.state <= tutState)
		{
			//Check if we need to wait
			if (camDef(tutPhase.wait) && tutPhase.wait > 0)
			{
				queue("addToConsole", tutPhase.wait * 1000); //wait is in seconds
				consoleVar[0].wait = 0;
				return;
			}

			if (camDef(tutPhase.func))
			{
				queue(tutPhase.func, 200);
			}

			if (camDef(tutPhase.audio))
			{
				playSound(tutPhase.audio, CAM_HUMAN_PLAYER);
			}

			if (camDef(tutPhase.clear) && tutPhase.clear)
			{
				clearConsole();
			}

			if (camDef(tutPhase.message))
			{
				console(tutPhase.message);
			}

			camTrace(tutState);
			consoleVar.shift();
		}

		queue("addToConsole", 600);
	}
}

function eventSelectionChanged(objects)
{
	var tut0 = tutState === 0 && consoleVar.length;
	var tut5 = tutState === 5 && consoleVar.length;

	if (tut0 || tut5)
	{
		//Check if they selected a truck.
		for (var i = 0, l = objects.length; i < l; ++i)
		{
			var obj = objects[i];
			if (obj.type === DROID && obj.droidType === DROID_CONSTRUCT)
			{
				if (tut0)
				{
					enableStructure("A0ResourceExtractor", CAM_HUMAN_PLAYER);
					increaseTutorialState();
				}
				else if (tut5 && obj.order !== DORDER_BUILD)
				{
					increaseTutorialState();
					checkHelpBuild();
				}
				break;
			}
		}
	}
}

function eventStructureBuilt(structure, droid)
{
	if (tutState === 1 && structure.stattype === RESOURCE_EXTRACTOR)
	{
		increaseTutorialState();
	}
	else if (tutState === 7 && structure.stattype === POWER_GEN)
	{
		//Maybe they did not understand instructions. Whatever the case, move on.
		if (!didTheyHelpBuildGen)
		{
			for (var i = 0; i < 3; ++i)
			{
				consoleVar.shift(); //skip ahead
			}
			tutState = 8;
		}
		else
		{
			increaseTutorialState();
		}
	}
	else if (tutState === 11 && structure.stattype === RESEARCH_LAB)
	{
		increaseTutorialState(); //If they did not make the other truck help build, move on.
	}
	else if (tutState === 22 && structure.stattype === FACTORY)
	{
		increaseTutorialState();
		//removeReticuleButton(BUILD_BUTTON);
	}
}

function eventGameInit()
{
	setTutorialMode(true);
}

//No way of winning if the extra victory callback does nothing.
function noWinningForYou() {}

function eventStartLevel()
{
	camSetStandardWinLossConditions(CAM_VICTORY_STANDARD, undefined, {
		callback: "noWinningForYou"
	});
	var startpos = getObject("startPosition");
	tutState = 0;
	didTheyHelpBuildGen = false;
	setUpConsoleAndAudioVar();

	centreView(startpos.x, startpos.y);
	setMissionTime(-1);
	setPower(500, CAM_HUMAN_PLAYER);
	hackStopIngameAudio();

	grantStartTech();
	setStructureLimits("A0LightFactory", 1, CAM_HUMAN_PLAYER);
	setStructureLimits("A0ResourceExtractor", 1, CAM_HUMAN_PLAYER);
	setStructureLimits("A0PowerGenerator", 1, CAM_HUMAN_PLAYER);
	setStructureLimits("A0ResearchFacility", 1, CAM_HUMAN_PLAYER);

	camSafeRemoveObject("mgArtifact", false);
	addFeature("OilResource", 24, 26);

	setMiniMap(true);
	setDesign(false);
	removeTemplate("ConstructionDroid");
	removeTemplate("ViperLtMGWheels");

	setReticuleButton(CLOSE_BUTTON, _("Close"), "image_cancel_up.png", "image_cancel_down.png");
	setReticuleButton(PRODUCTION_BUTTON, _("Manufacture - build factory first"), "", "");
	setReticuleButton(RESEARCH_BUTTON, _("Research - build research facility first"), "", "");
	setReticuleButton(BUILD_BUTTON, _("Build - manufacture constructor droids first"), "", "");
	setReticuleButton(DESIGN_BUTTON, _("Design (F4)"), "image_design_up.png", "image_design_down.png");
	setReticuleButton(INTEL_BUTTON, _("Intelligence Display (F5)"), "image_intelmap_up.png", "image_intelmap_down.png");
	setReticuleButton(COMMAND_BUTTON, _("Commanders - manufacture commanders first"), "", "");
	showInterface();

	queue("addToConsole", 2000);
}
